open Types

let drop (str: string) (n: int): string =
  let l = String.length str in
  String.sub str n (l - n)

let take (str: string) (n: int): string =
  String.sub str 0 n

let split (str: string) (ch: char): string list =
  let rec split' str l =
    try
      let i = String.index str ch in
      let t = take str i in
      let str' = drop str (i+1) in
      let l' = t::l in
      split' str' l'
    with Not_found ->
      List.rev (str::l)
  in
  split' str []

let split_at_last (l: 'a list) : ('a list * 'a) =
  let rec split_at_last_aux l acc =
    match l with
      | [] -> raise (Failure "split_at_last")
      | x::[] -> (List.rev acc, x)
      | x::xs -> split_at_last_aux xs (x::acc)
  in split_at_last_aux l []

(* Processes a line containing a clause. *)
let process_clause (line: string): clause =
  let line1 = if line.[0] = ' ' then drop line 1 else line in
  let foo sofar str = 
    let lit = 
      if str.[0] = '-' then (int_of_string (drop str 1), false)
      else (int_of_string str, true)
    in lit::sofar
  in 
  let (elems, zero) = split_at_last (split line1 ' ') in
  List.fold_left foo [] elems

let process_file (file: in_channel): formula  =
  let rec process_line (acc:formula) : formula =
    try
      let line = input_line file in
      if line = "" then acc
      else
        match line.[0] with
          | 'p' -> process_line acc
          | 'c' | '%' | '0' -> process_line acc
          | _ -> 
	    let c = process_clause line in
	    process_line (c::acc)
    with End_of_file -> acc
  in
  process_line []
