open Types
module HW = Homework

(* == Test formulas == *)
let phi0 = (* represents (v0 \/ not(v1)) /\ v1 *)
  let c0 = [(0, true); (1, false)]  in
  let c1 = [(1, true)] in
  [c0; c1]

let phi1 = 
  let c0 = [(0, true); (1, false)] in
  let c1 = [(0, false); (1, true)] in
  [c0; c1]

let phi2 = 
  let c0 = [(0, true); (1, false)] in
  let c1 = [(0, false); (0, false)] in
  let c2 = [(1, true)] in
  [c0; c1; c2]

let phi3 =
  let c0 = [(0, false); (1, true)] in
  let c1 = [(0, true); (1, false)] in
  [c0; c1]

(* == Pretty printing == *)
let string_of_lit (l: lit) : string =
  match l with
    | (n, true) -> "v" ^ string_of_int n
    | (n, false) -> "not(v" ^ string_of_int n ^ ")"
      
let rec string_of_clause (c: clause) : string =
  match c with
    | l::[] -> string_of_lit l
    | l::ls -> string_of_lit l ^ " \\/ " ^ string_of_clause ls
    | [] -> ""

let rec string_of_formula (f: formula) : string =
  match f with
    | c::[] -> "(" ^ string_of_clause c ^ ")"
    | c::cs -> "(" ^ (string_of_clause c) ^ ") /\\ " ^ (string_of_formula cs)
    | [] -> ""

let rec print_asgn (a: asgn) : string =
  let print_lit (l: lit) : string =
    match l with
      | (n, true) -> "v" ^ (string_of_int n) ^ " |-> true"
      | (n, false) -> "v" ^ (string_of_int n) ^ " |-> false"
  in match a with
    | l::[] -> print_lit l
    | l::ls -> print_lit l ^ ", " ^ print_asgn ls
    | [] -> ""
      
(* == The other functions == *)
let negate (l: lit) = 
  match l with
    | (v, true) -> (v, false)
    | (v, false) -> (v, true)

let choose_split (f: formula) : lit = 
  match f with
    | ((l::_)::_) -> l
    | _ -> failwith "choose_split called with empty formula"

let rec sat (fm:formula) : asgn option = 
  if List.length fm == 0 then Some []
  else
    (* match HW.unit_propagate fm with *)
    (*   | Some (fm',l) -> *)
    (* 	begin match sat fm' with *)
    (* 	  | Some al -> Some (l::al) *)
    (* 	  | None -> None *)
    (* 	end *)
    (*   | None -> *)
	let l = choose_split fm in
	begin match HW.set_formula fm l with
	  | Some fm' ->
	    begin match sat fm' with
	      | Some al -> Some (l::al)
	      | None ->
		begin match HW.set_formula fm (negate l) with
		  | Some fm' ->
		    begin match sat fm' with
		      | Some al -> Some ((negate l)::al)
		      | None -> None
		    end
		  | None -> None
		end
	    end
	  | None ->
	    begin match HW.set_formula fm (negate l) with
	      | Some fm' ->
		begin match sat fm' with
		  | Some al -> Some ((negate l)::al)
		  | None -> None
		end
	      | None -> None
	    end
	end

let sat_and_print (fm: formula) : unit =
  print_endline (string_of_formula fm);
  match sat fm with
    | Some al -> print_string "  sat\n"; print_endline (print_asgn al)
    | None -> print_string "  unsat\n" 

let main () = 
  let file_arg = ref "" in
  Arg.parse [] (fun str -> file_arg := str) "";
  if Array.length (Sys.argv) == 2 then
    let f = Parser.process_file (open_in !file_arg) in
    sat_and_print (List.rev f)
  else begin
    print_string "phi0: "; sat_and_print phi0;
    (* print_string "phi1: "; sat_and_print phi1; *)
    (* print_string "phi2: "; sat_and_print phi2; *)
    (* print_string "phi3: "; sat_and_print phi3 *)
  end
;;

main ();;
